@@ -9,27 +9,23 @@ Core driver with the low level functions.
 """
 
 # system imports
-from __future__ import absolute_import, division, print_function
-import sys
 import visa
 import logging
 import threading
 import numpy as np
 import time
+from threading import RLock
+from xdrlib import Error as XDRError
 
 # local import
 from keithley2600.keithley_doc import (CONSTANTS, FUNCTIONS, PROPERTIES,
-                                       CLASSES, PROPERTY_LISTS)
+                                       CLASSES, PROPERTY_LISTS, ALL_CMDS)
 from keithley2600.result_table import FETResultTable
 
-__version__ = 'v1.3.0'
+__version__ = 'v1.4.0'
 
-PY2 = sys.version[0] == '2'
 logger = logging.getLogger(__name__)
 
-if not PY2:
-    basestring = str  # in Python 3
-
 
 def log_to_screen(level=logging.DEBUG):
     log_to_stream(None, level)  # sys.stderr by default
@@ -50,18 +46,18 @@ def log_to_stream(stream_output, level=logging.DEBUG):
 class MagicPropertyList(object):
     """Mimics a Keithley TSP property list
 
-    Class which mimics a Keithley TSP property list and can be dynamically created. It
-    forwards all calls to the :func:`_read` method of the parent class and assignments
-    to the :func:`_write` method. Arbitrary values can be assigned, as long as
-    :func:`_write` can handle them.
+    Class which mimics a Keithley TSP property list and can be dynamically
+    created. It forwards all calls to the :func:`_read` method of the parent
+    class and assignments to the :func:`_write` method. Arbitrary values can be
+    assigned, as long as :func:`_write` can handle them.
 
-    This class is designed to look like a Keithley TSP "attribute" list, forward
-    function calls to the Keithley, and return the results.
+    This class is designed to look like a Keithley TSP "attribute" list,
+    forward function calls to the Keithley, and return the results.
 
     """
 
     def __init__(self, name, parent):
-        if not isinstance(name, basestring):
+        if not isinstance(name, str):
             raise ValueError('First argument must be of type str.')
         self._name = name
         self._parent = parent
@@ -99,18 +95,17 @@ class MagicPropertyList(object):
 class MagicFunction(object):
     """Mimics a Keithley TSP function
 
-    Class which mimics a function and can be dynamically created. It forwards all calls to
-    the :func:`_query` method of the parent class and returns the result from
-    :func:`_query`. Calls accept arbitrary arguments, as long as :func:`_query` can
-    handle them.
-
-    This class is designed to look like a Keithley TSP function, forward function calls to
-    the Keithley, and return the results.
+    Class which mimics a function and can be dynamically created. It forwards
+    all calls to the :func:`_query` method of the parent class and returns the
+    result from :func:`_query`. Calls accept arbitrary arguments, as long as
+    :func:`_query` can handle them.
 
+    This class is designed to look like a Keithley TSP function, forward
+    function calls to the Keithley, and return the results.
     """
 
     def __init__(self, name, parent):
-        if not isinstance(name, basestring):
+        if not isinstance(name, str):
             raise ValueError('First argument must be of type str.')
         self._name = name
         self._parent = parent
@@ -132,29 +127,26 @@ class MagicFunction(object):
 class MagicClass(object):
     """Mimics a TSP command group
 
-    Class which dynamically creates new attributes on access. These can be functions,
-    properties, or other classes.
+    Class which dynamically creates new attributes on access. These can be
+    functions, properties, or other classes.
 
-    MagicClass need the strings in FUNCTIONS and PROPERTIES to determine if the
-    accessed attribute should behave like a function or property. Otherwise,
-    it is assumed to be a new class.
-
-    Attribute setters and getters are forwarded to :func:`_write` and :func:`_query`
-    functions from the parent class. New functions are created as instances of
-    MagicFunction, new classes are created as instances of MagicClass.
+    Attribute setters and getters are forwarded to :func:`_write` and
+    :func:`_query` functions from the parent class. New functions are created
+    as instances of :class:`MagicFunction`, new classes are created as
+    instances of :class:`MagicClass`.
 
     MagicClass is designed to mimic a Keithley TSP command group with
     functions, attributes, and subordinate command groups.
 
     :Examples:
 
-    >>> inst = MagicClass('keithley')
-    >>> inst.reset()  # Dynamically creates a new attribute 'reset' as an instance
-    ...               # of MagicFunction, then calls it.
-    >>> inst.beeper  # Dynamically creates new attribute 'beeper' and sets it to
-    ...              # a new MagicClass instance.
-    >>> inst.beeper.enable  # Fakes the property 'enable' of 'beeper'
-    ...                     # with _write as setter and _query as getter.
+        >>> inst = MagicClass('keithley')
+        >>> inst.reset()  # Dynamically creates a new attribute 'reset' as an
+        ...               # instance of MagicFunction, then calls it.
+        >>> inst.beeper  # Dynamically creates new attribute 'beeper' and sets
+        ...              # it to a new MagicClass instance.
+        >>> inst.beeper.enable  # Fakes the property 'enable' of 'beeper'
+        ...                     # with _write as setter and _query as getter.
 
     """
 
@@ -162,7 +154,7 @@ class MagicClass(object):
     _parent = None
 
     def __init__(self, name, parent=None):
-        assert isinstance(name, basestring)
+        assert isinstance(name, str)
         self._name = name
         if parent is not None:
             self._parent = parent
@@ -188,10 +180,10 @@ class MagicClass(object):
         """Custom getter
 
         Creates an attribute as :class:`MagicClass`, :class:`MagicFunction` or
-        :class:`MagicPropertyList` instance if it is an expected Keithley TSP command
-        group, function or property list. Queries and returns the value if the attribute
-        corresponds to a Keithley TSP constant. Otherwise raises a
-        :class:`AttributeError`.
+        :class:`MagicPropertyList` instance if it is an expected Keithley TSP
+        command group, function or property list. Queries and returns the value
+        if the attribute corresponds to a Keithley TSP constant. Otherwise
+        raises a :class:`AttributeError`.
 
         :param str attr_name: Attribute name.
 
@@ -238,8 +230,8 @@ class MagicClass(object):
         :param str attr_name: Attribute name.
         :param value: Value to set.
 
-        :raises: :class:`ValueError` if trying to write a value to read-only Keithley
-            attributes.
+        :raises: :class:`ValueError` if trying to write a value to read-only
+            Keithley attributes.
         """
         if attr_name in PROPERTIES:
             value = self._convert_input(value)
@@ -274,54 +266,75 @@ class MagicClass(object):
     def __iter__(self):
         return self
 
-    def getdoc(self):
-        """Prevent pydoc from trying to document this class. This could
-        conflict with on-demand creation of attributes."""
-        pass
+    def __dir__(self):
+        prefix = self._name + '.' if self._name else ''
+
+        sub_cmds = (c.replace(prefix, '') for c in ALL_CMDS if c.startswith(prefix))
+        sub_cmds = list(set(c.split('.')[0] for c in sub_cmds))
+
+        sub_cmds += super().__dir__()
+
+        return sub_cmds
 
 
 class KeithleyIOError(Exception):
-    """Raised when trying to communicate with the Keithley while none is connected."""
+    """Raised when no Keithley instrument is connected."""
+    pass
+
+
+class KeithleyError(Exception):
+    """Raised for error messages from the Keithley itself."""
     pass
 
 
 class Keithley2600Base(MagicClass):
     """Keithley2600 driver
 
-    Keithley driver for base functionality. It replicates the functionality and syntax
-    from the Keithley TSP commands, which have a syntax similar to python. Attributes
-    are created on-access if they correspond to Keithley TSP type commands.
+    Keithley driver for base functionality. It replicates the functionality and
+    syntax from the Keithley TSP commands, which have a syntax similar to
+    python. Attributes are created on-access if they correspond to Keithley TSP
+    type commands.
 
     :param str visa_address: Visa address of the instrument.
-    :param str visa_library: PyVisa backend specification. Defaults to "@py" for pyvisa-py
-        but NI-VISA may be appropriate, depending on the interface type.
-    :param kwargs: Keyword arguments passed on to the visa connection, for instance
-        baude-rate or timeout. If not given, reasonable defaults will be used.
+    :param str visa_library: Path to visa library. Defaults to "@py" for pyvisa-py
+        but another IVI library may be appropriate (NI-VISA, Keysight VISA, R&S VISA,
+        tekVISA etc.). If an empty string is given, an IVI library will be used if
+        installed and pyvisa-py otherwise.
+    :param bool raise_keithley_errors: If ``True``, all Keithley errors will be
+        raised as Python errors instead of being ignored. This causes
+        significant communication overhead because the Keithley's error queue
+        is read after each command. Defaults to ``False``.
+    :param kwargs: Keyword arguments passed on to the visa connection, for
+        instance baude-rate or timeout. If not given, reasonable defaults will
+        be used.
 
     :cvar connection: Attribute holding a reference to the actual connection.
-    :cvar bool connected: `True` if connected to an instrument, `False` otherwise.
-    :cvar bool busy: `True` if a measurement is running, `False` otherwise.
-    :cvar list TO_TSP_LIST: List of python types which will be converted to Keithley TSP
-        lists by this driver and can be used as inputs. Currently, those are
-        :class:`list`, :class:`numpy.ndarray`, :class:`tuple`, :class:`set` and
-        :class:`range` (:class:`xrange` in Python 2).
-    :cvar int CHUNK_SIZE: Maximum length of lists which can be sent to the Keithley.
-        Longer lists will be transferred in chunks.
+    :cvar bool connected: ``True`` if connected to an instrument, ``False``
+        otherwise.
+    :cvar bool busy: ``True`` if a measurement is running, ``False`` otherwise.
+    :cvar list TO_TSP_LIST: List of python types which will be converted to
+        Keithley TSP lists by this driver and can be used as inputs. Currently,
+        those are :class:`list`, :class:`numpy.ndarray`, :class:`tuple`,
+        :class:`set` and :class:`range` (:class:`xrange` in Python 2).
+    :cvar int CHUNK_SIZE: Maximum length of lists which can be sent to the
+        Keithley. Longer lists will be transferred in chunks.
 
     .. note::
 
-        See the Keithley 2600 reference manual for all available commands and arguments.
-        Almost all Keithley TSP commands can be used with this driver. Not supported are:
+        See the Keithley 2600 reference manual for all available commands and
+        arguments. Almost all Keithley TSP commands can be used with this
+        driver. Not supported are:
 
          - ``lan.trigger[N].connected``: conflicts with the connected attribute
          - ``io.output()``: conflicts with ``smuX.source.output``
-         - All Keithley IV sweep commands. We implement our own in :class:`Keithley2600`.
+         - All Keithley IV sweep commands. We implement our own in
+           :class:`Keithley2600`.
 
-    :Example:
+    :Examples:
 
-    >>> keithley = Keithley2600Base('TCPIP0::192.168.2.121::INSTR')
-    >>> keithley.smua.measure.v()  # measures voltage at smuA
-    >>> keithley.smua.source.levelv = -40  # applies -40V to smuA
+        >>> keithley = Keithley2600Base('TCPIP0::192.168.2.121::INSTR')
+        >>> keithley.smua.measure.v()  # measures voltage at smuA
+        >>> keithley.smua.source.levelv = -40  # applies -40V to smuA
 
     """
 
@@ -329,10 +342,12 @@ class Keithley2600Base(MagicClass):
     connected = False
     busy = False
 
-    TO_TSP_LIST = (list, np.ndarray, tuple, set, xrange if PY2 else range)
+    TO_TSP_LIST = (list, np.ndarray, tuple, set, range)
     CHUNK_SIZE = 50
 
-    def __init__(self, visa_address, visa_library='@py', **kwargs):
+    _lock = RLock()
+
+    def __init__(self, visa_address, visa_library='@py', raise_keithley_errors=False, **kwargs):
 
         MagicClass.__init__(self, name='', parent=self)
         self._name = ''
@@ -341,6 +356,9 @@ class Keithley2600Base(MagicClass):
 
         self.visa_address = visa_address
         self.visa_library = visa_library
+        self._connection_kwargs = kwargs
+
+        self.raise_keithley_errors = raise_keithley_errors
 
         # open visa resource manager with selected library / backend
         self.rm = visa.ResourceManager(self.visa_library)
@@ -361,8 +379,7 @@ class Keithley2600Base(MagicClass):
         :param kwargs: Keyword arguments for Visa connection.
 
         """
-        connection_error = OSError if PY2 else ConnectionError
-        # noinspection PyBroadException
+        kwargs = kwargs or self._connection_kwargs  # use specified or remembered kwargs
         try:
             self.connection = self.rm.open_resource(self.visa_address, **kwargs)
             self.connection.read_termination = '\n'
@@ -372,7 +389,7 @@ class Keithley2600Base(MagicClass):
             self.connection = None
             self.connected = False
             raise
-        except connection_error:
+        except ConnectionError:
             logger.info('Connection error. Please check that ' +
                         'no other program is connected.')
             self.connection = None
@@ -409,36 +426,64 @@ class Keithley2600Base(MagicClass):
         """
         Writes text to Keithley. Input must be a string.
         """
-        logger.debug('write: %s' % value)
 
-        if self.connection:
-            self.connection.write(value)
-        else:
-            raise KeithleyIOError(
-                'No connection to keithley present. Try to call connect().')
+        with self._lock:
+            logger.debug('write: %s' % value)
+
+            if self.connection:
+
+                if self.raise_keithley_errors and 'errorqueue' not in value:
+                    self.errorqueue.clear()
+
+                self.connection.write(value)
+
+                if self.raise_keithley_errors and 'errorqueue' not in value:
+                    err = self.errorqueue.next()
+                    if err[0] != 0:
+                        raise KeithleyError("Error during command '{0}': {1}".format(
+                            value, err[1]))
+            else:
+                raise KeithleyIOError(
+                    "No connection to keithley present. Try to call 'connect'.")
 
     def _query(self, value):
         """
         Queries and expects response from Keithley. Input must be a string.
         """
-        logger.debug('write: print(%s)' % value)
 
-        if self.connection:
-            r = self.connection.query('print(%s)' % value)
-            logger.debug('read: %s' % r)
+        with self._lock:
+            logger.debug('write: print(%s)' % value)
+            if self.connection:
 
-            return self._parse_response(r)
-        else:
-            raise KeithleyIOError(
-                'No connection to keithley present. Try to call connect().')
+                if self.raise_keithley_errors and 'errorqueue' not in value:
+                    self.errorqueue.clear()
+
+                try:
+                    r = self.connection.query('print(%s)' % value)
+                    logger.debug('read: %s' % r)
+                except XDRError:
+                    r = 'nil'
+                    logger.debug('read failed: unpack-error')
+
+                if self.raise_keithley_errors and 'errorqueue' not in value:
+                    err = self.errorqueue.next()
+                    if err[0] != 0:
+                        err_msg = err[1].replace('TSP Runtime error at line 1: ', '')
+                        raise KeithleyError("Error during command '{0}': {1}".format(
+                            value, err_msg))
+
+                return self._parse_response(r)
+            else:
+                raise KeithleyIOError(
+                    "No connection to keithley present. Try to call 'connect'.")
 
     @staticmethod
     def _parse_single_response(string):
 
         # Dictionary to convert from Keithley TSP to Python types.
         # Note that emtpy strings are converted to `None`. This is necessary
-        # since `self.connection.query('print(myfunc())')` returns an empty
-        # string if the TSP function `myfunc()` returns 'nil'.
+        # since `self.connection.query('print(func())')` returns an empty
+        # string if the TSP function `func()` returns 'nil'.
         conversion_dict = {'true': True, 'false': False, 'nil': None, '': None}
 
         try:
@@ -483,59 +528,69 @@ class Keithley2600Base(MagicClass):
 class Keithley2600(Keithley2600Base):
     """Keithley2600 driver with high level functionality
 
-    Keithley driver with access to base functions and higher level functions such as IV
-    measurements, transfer and output curves, etc. Inherits from
-    :class:`Keithley2600Base`. Base commands replicate the functionality and syntax
-    of Keithley TSP functions.
+    Keithley driver with access to base functions and higher level functions
+    such as IV measurements, transfer and output curves, etc. Inherits from
+    :class:`Keithley2600Base`. Base commands replicate the functionality and
+    syntax of Keithley TSP functions.
 
     :param str visa_address: Visa address of the instrument.
-    :param str visa_library: PyVisa backend specification. Defaults to "@py" for pyvisa-py
-        but NI-VISA may be appropriate, depending on the interface type.
-    :param kwargs: Keyword arguments passed on to the visa connection, for instance
-        baude-rate or timeout. If not given, reasonable defaults will be used.
+    :param str visa_library: Path to visa library. Defaults to "@py" for pyvisa-py
+        but another IVI library may be appropriate (NI-VISA, Keysight VISA, R&S VISA,
+        tekVISA etc.). If an empty string is given, an IVI library will be used if
+        installed and pyvisa-py otherwise.
+    :param bool raise_keithley_errors: If ``True``, all Keithley errors will be
+        raised as Python errors instead of being ignored. This causes
+        significant communication overhead because the Keithley's error queue
+        is read after each command. Defaults to ``False``.
+    :param kwargs: Keyword arguments passed on to the visa connection, for
+        instance baude-rate or timeout. If not given, reasonable defaults will
+        be used.
 
     :cvar connection: Attribute holding a reference to the actual connection.
-    :cvar bool connected: `True` if connected to an instrument, `False` otherwise.
-    :cvar bool busy: `True` if a measurement is running, `False` otherwise.
+    :cvar bool connected: ``True`` if connected to an instrument, ``False``
+        otherwise.
+    :cvar bool busy: ``True`` if a measurement is running, ``False`` otherwise.
 
-    :Example:
+    :Examples:
 
-    *Base commands from Keithley TSP*:
+        *Base commands from Keithley TSP*:
 
-    >>> k = Keithley2600('TCPIP0::192.168.2.121::INSTR')
-    >>> volts = k.smua.measure.v()  # measures and returns the smuA voltage
-    >>> k.smua.source.levelv = -40  # sets source level of smuA
-    >>> k.smua.nvbuffer1.clear()  # clears nvbuffer1 of smuA
+        >>> k = Keithley2600('TCPIP0::192.168.2.121::INSTR')
+        >>> volts = k.smua.measure.v()  # measures and returns the smuA voltage
+        >>> k.smua.source.levelv = -40  # sets source level of smuA
+        >>> k.smua.nvbuffer1.clear()  # clears nvbuffer1 of smuA
 
-    *New mid-level commands*:
+        *New mid-level commands*:
 
-    >>> data = k.readBuffer(k.smua.nvbuffer1)
-    >>> errs = k.readErrorQueue()
-    >>> k.setIntegrationTime(k.smua, 0.001) # in sec
+        >>> data = k.readBuffer(k.smua.nvbuffer1)
+        >>> errs = k.readErrorQueue()
+        >>> k.setIntegrationTime(k.smua, 0.001) # in sec
 
-    >>> k.applyVoltage(k.smua, -60) # applies -60V to smuA
-    >>> k.applyCurrent(k.smub, 0.1) # sources 0.1A from smuB
-    >>> k.rampToVoltage(k.smua, 10, delay=0.1, step_size=1)
+        >>> k.applyVoltage(k.smua, -60) # applies -60V to smuA
+        >>> k.applyCurrent(k.smub, 0.1) # sources 0.1A from smuB
+        >>> k.rampToVoltage(k.smua, 10, delay=0.1, step_size=1)
 
-    >>> # voltage sweeps, single and dual SMU
-    >>> k.voltageSweepSingleSMU(smu=k.smua, smu_sweeplist=range(0, 61),
-    ...                         t_int=0.1, delay=-1, pulsed=False)
-    >>> k.voltageSweepDualSMU(smu1=k.smua, smu2=k.smub,
-    ...                       smu1_sweeplist=range(0, 61),
-    ...                       smu2_sweeplist=range(0, 61),
-    ...                       t_int=0.1, delay=-1, pulsed=False)
+        >>> # voltage sweeps, single and dual SMU
+        >>> k.voltageSweepSingleSMU(smu=k.smua, smu_sweeplist=range(0, 61),
+        ...                         t_int=0.1, delay=-1, pulsed=False)
+        >>> k.voltageSweepDualSMU(smu1=k.smua, smu2=k.smub,
+        ...                       smu1_sweeplist=range(0, 61),
+        ...                       smu2_sweeplist=range(0, 61),
+        ...                       t_int=0.1, delay=-1, pulsed=False)
 
-    *New high-level commands*:
+        *New high-level commands*:
 
-    >>> data1 = k.outputMeasurement(...) # records output curve
-    >>> data2 = k.transferMeasurement(...) # records transfer curve
+        >>> data1 = k.outputMeasurement(...) # records output curve
+        >>> data2 = k.transferMeasurement(...) # records transfer curve
 
     """
 
     SMU_LIST = ['smua', 'smub']
 
-    def __init__(self, visa_address, visa_library='@py', **kwargs):
-        Keithley2600Base.__init__(self, visa_address, visa_library, **kwargs)
+    def __init__(self, visa_address, visa_library='@py', raise_keithley_errors=False,
+                 **kwargs):
+        Keithley2600Base.__init__(self, visa_address, visa_library,
+                                  raise_keithley_errors=raise_keithley_errors, **kwargs)
 
     def __repr__(self):
         return '<%s(%s)>' % (type(self).__name__, self.visa_address)
@@ -599,9 +654,10 @@ class Keithley2600(Keithley2600Base):
         Sets the integration time of SMU for measurements in sec.
 
         :param smu: A keithley smu instance.
-        :param float t_int: Integration time in sec. Value must be between 0.001
-            and 25 power line cycles (50Hz or 60 Hz).
-        :raises: :class:`ValueError` for too short or too long integration times.
+        :param float t_int: Integration time in sec. Value must be between
+            1/1000 and 25 power line cycles (50Hz or 60 Hz).
+        :raises: :class:`ValueError` for too short or too long integration
+            times.
         """
 
         self._check_smu(smu)
@@ -626,6 +682,7 @@ class Keithley2600(Keithley2600Base):
         self._check_smu(smu)
 
         smu.source.levelv = voltage
+        smu.source.func = smu.OUTPUT_DCVOLTS
         smu.source.output = smu.OUTPUT_ON
 
     def applyCurrent(self, smu, curr):
@@ -638,12 +695,15 @@ class Keithley2600(Keithley2600Base):
         self._check_smu(smu)
 
         smu.source.leveli = curr
+        smu.source.func = smu.OUTPUT_DCAMPS
         smu.source.output = smu.OUTPUT_ON
 
     def measureVoltage(self, smu):
         """
         Measures a voltage at the specified SMU.
 
+        :param smu: A keithley smu instance.
+
         :returns: Measured voltage in Volts.
         :rtype: float
         """
@@ -656,6 +716,8 @@ class Keithley2600(Keithley2600Base):
         """
         Measures a current at the specified SMU.
 
+        :param smu: A keithley smu instance.
+
         :returns: Measured current in Ampere.
         :rtype: float
         """
@@ -687,32 +749,33 @@ class Keithley2600(Keithley2600Base):
 
         step = np.sign(target_volt - vcurr) * abs(step_size)
 
-        for v in np.arange(vcurr, target_volt + step, step):
+        for v in np.arange(vcurr, target_volt, step):
             smu.source.levelv = v
             smu.measure.v()
             time.sleep(delay)
 
-        target_volt = smu.measure.v()
+        smu.source.levelv = target_volt
         logger.info('Gate voltage set to Vg = %s V.' % round(target_volt))
 
         self.beeper.beep(0.3, 2400)
 
     def voltageSweepSingleSMU(self, smu, smu_sweeplist, t_int, delay, pulsed):
         """
-        Sweeps the voltage through the specified list of steps at the given SMU. Measures
-        and returns the current and voltage during the sweep.
+        Sweeps the voltage through the specified list of steps at the given
+        SMU. Measures and returns the current and voltage during the sweep.
 
         :param smu: A keithley smu instance.
-        :param smu_sweeplist: Voltages to sweep through (can be a numpy array, list,
-            tuple or range / xrange).
-        :param float t_int: Integration time per data point. Must be
-            between 0.001 to 25 times the power line frequency (50Hz or 60Hz).
-        :param float delay: Settling delay before measurement. Set ``delay = -1`` for
-            an automatic measurement once the current is stable.
-        :param bool pulsed: `True` or `False` for pulsed or continuous sweep.
+        :param smu_sweeplist: Voltages to sweep through (can be a numpy array,
+            list, tuple or range / xrange).
+        :param float t_int: Integration time per data point. Must be between
+            0.001 to 25 times the power line frequency (50Hz or 60Hz).
+        :param float delay: Settling delay before each measurement. A value of
+            -1 automatically starts a measurement once the current is stable.
+        :param bool pulsed: Select pulsed or continuous sweep. In a pulsed
+            sweep, the voltage is always reset to zero between data points.
 
-        :returns: Lists of voltages and currents measured during the sweep (in Volt and
-            Ampere, respectively): ``(v_smu, i_smu)``.
+        :returns: Lists of voltages and currents measured during the sweep (in
+            Volt and Ampere, respectively): ``(v_smu, i_smu)``.
         :rtype: (list, list)
         """
 
@@ -722,7 +785,8 @@ class Keithley2600(Keithley2600Base):
 
         # set state to busy
         self.busy = True
-        # Define lists containing results. If we abort early, we have something to return.
+        # Define lists containing results.
+        # If we abort early, we have something to return.
         v_smu, i_smu = [], []
 
         if self.abort_event.is_set():
@@ -885,8 +949,8 @@ class Keithley2600(Keithley2600Base):
     def voltageSweepDualSMU(self, smu1, smu2, smu1_sweeplist, smu2_sweeplist, t_int,
                             delay, pulsed):
         """
-        Sweeps voltages at two SMUs. Measures and returns current and voltage during
-        sweep.
+        Sweeps voltages at two SMUs. Measures and returns current and voltage
+        during sweep.
 
         :param smu1: 1st keithley smu instance to be swept.
         :param smu2: 2nd keithley smu instance to be swept.
@@ -896,12 +960,14 @@ class Keithley2600(Keithley2600Base):
              array, list, tuple, range / xrange).
         :param float t_int: Integration time per data point. Must be
             between 0.001 to 25 times the power line frequency (50Hz or 60Hz).
-        :param float delay: Settling delay before measurement. Set ``delay = -1`` for
-            an automatic measurement once the current is stable.
-        :param bool pulsed: `True` or `False` for pulsed or continuous sweep.
+        :param float delay: Settling delay before each measurement. A value of
+            -1 automatically starts a measurement once the current is stable.
+        :param bool pulsed: Select pulsed or continuous sweep. In a pulsed
+            sweep, the voltage is always reset to zero between data points.
 
-        :returns: Lists of voltages and currents measured during the sweep (in Volt and
-            Ampere, respectively): ``(v_smu1, i_smu1, v_smu2, i_smu2)``.
+        :returns: Lists of voltages and currents measured during the sweep (in
+            Volt and Ampere, respectively): ``(v_smu1, i_smu1, v_smu2,
+            i_smu2)``.
         :rtype: (list, list, list, list)
         """
 
@@ -915,7 +981,8 @@ class Keithley2600(Keithley2600Base):
 
         # set state to busy
         self.busy = True
-        # Define lists containing results. If we abort early, we have something to return.
+        # Define lists containing results.
+        # If we abort early, we have something to return.
         v_smu1, i_smu1, v_smu2, i_smu2 = [], [], [], []
 
         if self.abort_event.is_set():
@@ -1112,8 +1179,8 @@ class Keithley2600(Keithley2600Base):
     def transferMeasurement(self, smu_gate, smu_drain, vg_start, vg_stop,
                             vg_step, vd_list, t_int, delay, pulsed):
         """
-        Records a transfer curve with forward and reverse sweeps and returns the
-        results in a :class:`sweep_data.TransistorSweepData` instance.
+        Records a transfer curve with forward and reverse sweeps and returns
+        the results in a :class:`sweep_data.TransistorSweepData` instance.
 
 
         :param smu_gate: Keithley smu attached to gate electrode.
@@ -1125,9 +1192,10 @@ class Keithley2600(Keithley2600Base):
              array, list, tuple, range / xrange.
         :param float t_int: Integration time per data point. Must be
             between 0.001 to 25 times the power line frequency (50Hz or 60Hz).
-        :param float delay: Settling delay before measurement. Set ``delay = -1`` for
-            an automatic measurement once the current is stable.
-        :param bool pulsed: `True` or `False` for pulsed or continuous sweep.
+        :param float delay: Settling delay before each measurement. A value of
+            -1 automatically starts a measurement once the current is stable.
+        :param bool pulsed: Select pulsed or continuous sweep. In a pulsed
+            sweep, the voltage is always reset to zero between data points.
 
         :returns: Transfer curve data.
         :rtype: :class:`sweep_data.TransistorSweepData`
@@ -1147,8 +1215,14 @@ class Keithley2600(Keithley2600Base):
         sweeplist_gate = np.append(sweeplist_gate_fwd, sweeplist_gate_rvs)
 
         # create ResultTable instance
-        params = {'sweep_type': 'transfer', 't_int': t_int, 'delay': delay,
-                  'pulsed': pulsed}
+        params = {
+            'sweep_type': 'transfer',
+            'time': time.time(),
+            'time_str': time.strftime('%d/%m/%Y %H:%M'),
+            't_int': t_int,
+            'delay': delay,
+            'pulsed': pulsed,
+        }
         rt = FETResultTable(params=params)
         rt.append_column(sweeplist_gate, name='Gate voltage', unit='V')
 
@@ -1200,9 +1274,10 @@ class Keithley2600(Keithley2600Base):
              array, list, tuple, range / xrange.
         :param float t_int: Integration time per data point. Must be
             between 0.001 to 25 times the power line frequency (50Hz or 60Hz).
-        :param float delay: Settling delay before measurement. Set ``delay = -1`` for
-            an automatic measurement once the current is stable.
-        :param bool pulsed: `True` or `False` for pulsed or continuous sweep.
+        :param float delay: Settling delay before each measurement. A value of
+            -1 automatically starts a measurement once the current is stable.
+        :param bool pulsed: Select pulsed or continuous sweep. In a pulsed
+            sweep, the voltage is always reset to zero between data points.
 
         :returns: Output curve data.
         :rtype: :class:`sweep_data.TransistorSweepData`
@@ -1222,8 +1297,14 @@ class Keithley2600(Keithley2600Base):
         sweeplist_drain = np.append(sweeplist_drain_fwd, sweeplist_drain_rvs)
 
         # create ResultTable instance
-        params = {'sweep_type': 'output', 't_int': t_int, 'delay': delay,
-                  'pulsed': pulsed}
+        params = {
+            'sweep_type': 'output',
+            'time': time.time(),
+            'time_str': time.strftime('%d/%m/%Y %H:%M'),
+            't_int': t_int,
+            'delay': delay,
+            'pulsed': pulsed,
+        }
         rt = FETResultTable(params=params)
         rt.append_column(sweeplist_drain, name='Drain voltage', unit='V')
 
@@ -1244,9 +1325,9 @@ class Keithley2600(Keithley2600Base):
 
             if not self.abort_event.is_set():
                 i_s = np.array(i_d) + np.array(i_g)
-                rt.append_column(i_s, name='Source current (Vd = %s)' % vgate, unit='A')
-                rt.append_column(i_d, name='Drain current (Vd = %s)' % vgate, unit='A')
-                rt.append_column(i_g, name='Gate current (Vd = %s)' % vgate, unit='A')
+                rt.append_column(i_s, name='Source current (Vg = %s)' % vgate, unit='A')
+                rt.append_column(i_d, name='Drain current (Vg = %s)' % vgate, unit='A')
+                rt.append_column(i_g, name='Gate current (Vg = %s)' % vgate, unit='A')
 
         self.reset()
         self.beeper.beep(0.3, 2400)
@@ -1257,11 +1338,12 @@ class Keithley2600(Keithley2600Base):
     def playChord(self, notes=('C6', 'E6', 'G6'), durations=0.3):
         """Plays a chord on the Keithley.
 
-        :param list notes: List of notes in scientific pitch notation, for instance
-            ``['F4', 'Ab4', 'C4']`` for a f-minor chord in the 4th octave. Defaults to
-            c-major in the 6th octave.
-        :param durations: List of durations for each note in sec. If a single float is
-            given, all notes will have the same duration. Defaults to 0.3 sec.
+        :param list notes: List of notes in scientific pitch notation, for
+            instance ``['F4', 'Ab4', 'C4']`` for a f-minor chord in the 4th
+            octave. Defaults to c-major in the 6th octave.
+        :param durations: List of durations for each note in sec. If a single
+            float is given, all notes will have the same duration. Defaults to
+            0.3 sec.
         :type durations: float or list
         """
 
@@ -1310,14 +1392,15 @@ class Keithley2600Factory(object):
 
     def __new__(cls, *args, **kwargs):
         """
-        Create new instance for a new visa_address, otherwise return existing instance.
+        Create new instance for a new visa_address, otherwise return existing
+        instance.
         """
         if args[0] in cls._instances:
-            logger.debug('Returning existing instance with address %s.' % args[0])
+            logger.debug("Returning existing instance with address '%s'." % args[0])
 
             return cls._instances[args[0]]
         else:
-            logger.debug('Creating new instance with address %s.' % args[0])
+            logger.debug("Creating new instance with address '%s'." % args[0])
             instance = Keithley2600(*args, **kwargs)
             cls._instances[args[0]] = instance
 
