@@ -9,16 +9,10 @@ Submodule defining classes to store, plot, and save measurement results.
 
 """
 
-import sys
 import os
 import re
 import numpy as np
 
-PY2 = sys.version[0] == '2'
-
-if not PY2:
-    basestring = str  # in Python 3
-
 
 def find_numbers(string):
 
@@ -80,7 +74,7 @@ class ResultTable(object):
     :type data: numpy.ndarray or NoneType
     :param dict params: Dictionary of measurement parameters.
 
-    :Example:
+    :Examples:
 
         Create a :class:`ResultTable` to hold current-vs-time data:
 
@@ -172,7 +166,7 @@ class ResultTable(object):
     @column_names.setter
     def column_names(self, names_list):
         """Setter: List of strings with column names."""
-        if not all(isinstance(x, basestring) for x in names_list):
+        if not all(isinstance(x, str) for x in names_list):
             raise ValueError("All column names must be of type 'str'.")
         elif not len(names_list) == self.ncols:
             raise ValueError('Number of column names does not match number of columns.')
@@ -188,7 +182,7 @@ class ResultTable(object):
     @column_units.setter
     def column_units(self, units_list):
         """Setter: List of strings with column units."""
-        if not all(isinstance(x, basestring) for x in units_list):
+        if not all(isinstance(x, str) for x in units_list):
             raise ValueError("All column_units must be of type 'str'.")
         elif not len(units_list) == self.ncols:
             raise ValueError('Number of column_units does not match number of columns.')
@@ -525,8 +519,11 @@ class ResultTable(object):
         :param list y_clmns: List of column numbers or column names for y-axis data. If
             not given, all columns will be plotted against the x-axis column.
         :param function func: Function to apply to y-data before plotting.
-        :param bool live: If ``True``, update plot when new rows are added (default:
-            ``False``).
+        :param bool live: If ``True``, update the plot when new data is added (default:
+            ``False``). Plotting will be carried out in the main (GUI) thread, therefore
+            take care not to block the thread. This can be achieved for instance by adding
+            data in a background thread which carries out the measurement, or by calling
+            `matplotlib.pyplot.pause` after adding data to give the GUI time to update.
 
         :returns: :class:`ResultTablePlot` instance with Matplotlib figure.
         :rtype: :class:`ResultTablePlot`
@@ -707,8 +704,11 @@ class ResultTablePlot(object):
         all columns will be plotted against the x-axis column.
     :type y_clmns: list(int or str)
     :param function func: Function to apply to y-data before plotting.
-    :param bool live: If ``True``, update plot when new rows are added. Default to
-        `False``.
+    :param bool live: If ``True``, update the plot when new data is added (default:
+        ``False``). Plotting will be carried out in the main (GUI) thread, therefore
+        take care not to block the thread. This can be achieved for instance by adding
+        data in a background thread which carries out the measurement, or by calling
+        `matplotlib.pyplot.pause` after adding data to give the GUI time to update.
     """
 
     def __init__(self, result_table, x_clmn=0, y_clmns=None, func=lambda x: x,
@@ -772,14 +772,21 @@ class ResultTablePlot(object):
 
         if live and matplotlib.get_backend() == 'Qt5Agg':
             self._timer = self.fig.canvas.new_timer()
-            self._timer.add_callback(self.update_plot)
+            self._timer.add_callback(self.update)
             self._timer.start(100)
 
     def show(self):
+        """
+        Shows the plot.
+        """
 
         self.fig.show()
 
-    def update_plot(self):
+    def update(self):
+        """
+        Updates the plot with the data of the corresponding :class:`ResultTable`.
+        This will be called periodically when :param:``live`` is ``True``.
+        """
 
         x = self.result_table.data[:, self.x_clmn]
 
